/* $Header: /nw/tony/src/stevie/src/RCS/mark.c,v 1.3 89/03/11 22:42:39 tony $
 *
 *  Routines to save and retrieve marks.
 *
 *  10/12/90 bowles: Cleaned up code; allow only marks 'a' through 'z'.
 *    Assumes letters 'a' through 'z' are contiguous (i.e., 'b' = 'a' + 1),
 *    which is true for ASCII.  Not sure if this applies to EBCDIC.
 */

#include "stevie.h"


/* Total number of marks, including the previous context mark */
#define NMARKS    (('z' - 'a' + 1) + 1)

/*
 *  Mark type.
 */
typedef struct {
	LPTR	pos;		/* position in file */
	bool_t	valid;		/* TRUE if this mark is valid */
} Mark;

/*
 *  Table of all marks.  Positions 0 through (NMARKS-2) contain the 
 *  letter marks 'a' through 'z'.  The last position is used for the 
 *  previous context mark.
 */
static Mark 	Marklist[NMARKS];
static Mark 	*Prevcontext = &(Marklist[NMARKS - 1]);

/*
 *  Return a pointer to a mark 'c'.  Assumes 'c' is a valid mark character 
 *  (i.e., it is a lower case letter).
 */
#define get_markptr(c)     &(Marklist[ (c)-'a' ])

/*
 *  Returns TRUE if 'c' is a valid mark character.
 */
#define valid_mark_char(c)   (isalpha(c) && islower(c))


/*
 * setmark(c) - set mark 'c' at current cursor position
 *
 * Returns TRUE on success, FALSE if bad mark name given.
 */
PUBLIC
bool_t setmark(c)
	register int	c;
{
	register Mark 	*m;

#ifdef KANJI
	if (c & 0xff80)
		return FALSE;
#endif

	if (! valid_mark_char(c))
		return FALSE;

	/*
	 * Assign the cursor position to corresponding mark.
	 */
	m = get_markptr(c);
	m->valid = TRUE;
	m->pos   = *Curschar;
	
	return(TRUE);
}


/*
 * setpcmark() - set the previous context mark to the current position
 */
PUBLIC
void setpcmark()
{
	Prevcontext->pos   = *Curschar;
	Prevcontext->valid = TRUE;
}


/*
 * getmark(c) - find mark for character 'c'
 *
 * Return pointer to LPTR or NOLPTR if no such mark.
 */
PUBLIC
LPTR *getmark(c)
	register int	c;
{
	register Mark	*m;

#ifdef KANJI
	if (c & 0xff80)
		return(NOLPTR);
#endif

#ifdef NOBACKQ
	if (c == '\'' || c == '`' || c == '\\')	/* previous context mark */
#else
	if (c == '\'' || c == '`')	/* previous context mark */
#endif
	{
		m = Prevcontext;
	} else if (! valid_mark_char(c)) {
		return(NOLPTR);
	} else {
		m = get_markptr(c);
	}
	return(m->valid  ?  &(m->pos)  :  NOLPTR);
}


/*
 * clrall() - clear all marks
 *
 * Used mainly when trashing the entire buffer during ":e" type commands
 */
PUBLIC
void clrall()
{
	register Mark *m;

	for (m = Marklist;  m < (Marklist + NMARKS);  m++) {
		m->valid = FALSE;
	}
}


/*
 * clrmark(line) - clear any marks for 'line'
 *
 * Used any time a line is deleted so we don't have marks pointing to
 * non-existent lines.
 */
PUBLIC
void clrmark(line)
	register LINE	*line;
{
	register Mark	*m;

	for (m = Marklist;  m < (Marklist + NMARKS);  m++) {
		if (m->valid && (m->pos.linep == line)) {
			m->valid = FALSE;
		}
	}
}

